<?php

namespace Drupal\llms_txt_generator\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Cache\CacheableResponse;
use Drupal\llms_txt_generator\Service\LlmsFileGenerator;

/**
 * Returns the content of the llms.txt file.
 */
class LlmsTextGeneratorController extends ControllerBase {

  /**
   * The LLMs file generator service.
   *
   * @var \Drupal\llms_txt_generator\Service\LlmsFileGenerator
   */
  protected $fileGenerator;

  /**
   * Constructs a new LlmsTextGeneratorController.
   *
   * @param \Drupal\llms_txt_generator\Service\LlmsFileGenerator $file_generator
   *   The LLMs file generator service.
   */
  public function __construct(LlmsFileGenerator $file_generator) {
    $this->fileGenerator = $file_generator;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('llms_txt_generator.file_generator')
    );
  }

  /**
   * Returns the content of the llms.txt file.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   A plain text response or a 404 if not available.
   */
  public function content(): Response {
    $config = $this->config('llms_txt_generator.settings');

    // If disabled via config, return 404.
    if ($config->get('enable_file') === FALSE) {
      return new Response('', 404);
    }

    // Attempt to get the file content from the generator.
    $content = $this->fileGenerator->getLlmsFileContent();

    // Fallback to config value if file missing.
    if ($content === FALSE) {
      $content = $config->get('llms_txt_content');
    }

    // Still nothing, Return 404.
    if (empty(trim($content))) {
      return new Response('', 404);
    }


    // Build cacheable response with appropriate headers.
    $response = new CacheableResponse($content, Response::HTTP_OK, [
      'Content-Type' => 'text/plain; charset=UTF-8',
      'X-Robots-Tag' => 'noindex',
      // Set client-side caching (browser/CDN).
      'Cache-Control' => 'public, max-age=3600',
    ]);


    // Add cache metadata so Drupal can invalidate it when config changes.
    $metadata = $response->getCacheableMetadata();
    $metadata->addCacheTags(['config:llms_txt_generator.settings']);
    $metadata->addCacheContexts(['url.site']);

    return $response;
  }

}
