<?php

namespace Drupal\llms_txt_generator\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\file\FileRepositoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Service responsible for generating and accessing the llms.txt file.
 */
class LlmsFileGenerator implements LlmsFileGeneratorInterface {

  /**
   * The URI of the llms.txt file.
   */
  const LLMS_TXT_URI = 'public://llms.txt';

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The file repository service.
   *
   * @var \Drupal\file\FileRepositoryInterface
   */
  protected $fileRepository;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new LlmsFileGenerator.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\file\FileRepositoryInterface $file_repository
   *   The file repository service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    FileSystemInterface $file_system,
    LoggerChannelFactoryInterface $logger_factory,
    FileRepositoryInterface $file_repository,
    ModuleHandlerInterface $module_handler,
    RequestStack $request_stack
  ) {
    $this->configFactory = $config_factory;
    $this->fileSystem = $file_system;
    $this->logger = $logger_factory->get('llms_txt_generator');
    $this->fileRepository = $file_repository;
    $this->moduleHandler = $module_handler;
    $this->requestStack = $request_stack;
  }

  /**
   * Generates the llms.txt file based on configuration.
   *
   * @param bool $suppress_empty_warning
   *   Whether to suppress warnings about empty content.
   *
   * @return bool
   *   TRUE if file was successfully generated, FALSE otherwise.
   */
  public function generateLlmsFile($suppress_empty_warning = FALSE) {
    $config = $this->configFactory->get('llms_txt_generator.settings');

    if ($config->get('enable_file') === FALSE) {
      $this->logger->notice('llms.txt file generation is disabled in the configuration.');
      return FALSE;
    }

    $content = $config->get('llms_txt_content');

    if (empty(trim($content))) {
      if (!$suppress_empty_warning) {
        $this->logger->warning('llms.txt generation skipped: the file content is empty.');
      }
      return FALSE;
    }

    try {
      $directory = 'public://';
      // Check for directory permissions first
      if (!$this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY)) {
        $this->logger->warning(
          'Cannot create or write to directory @dir due to permission issues.',
          ['@dir' => $directory]
        );
        return FALSE;
      }

      $uri = self::LLMS_TXT_URI;

      // Remove existing llms.txt to replace it with a fresh version.
      if (file_exists($uri)) {
        try {
          $this->fileSystem->delete($uri);
        }
        catch (\Exception $e) {
          $this->logger->warning(
            'Cannot delete existing llms.txt file due to permission issues: @message',
            ['@message' => $e->getMessage()],
          );
          return FALSE;
        }
      }

      // Write the new llms.txt file.
      $file = $this->fileRepository->writeData($content, $uri, FileSystemInterface::EXISTS_REPLACE);

      if ($file) {
        $this->logger->notice('llms.txt file was successfully generated at @uri.', ['@uri' => $uri]);
        return TRUE;
      }

      $this->logger->warning('Failed to save llms.txt file due to permission issues.');
      return FALSE;
    }
    catch (\Exception $e) {
      $this->logger->warning(
        'Error generating llms.txt file due to permission issues: @message',
        ['@message' => $e->getMessage()],
      );
      return FALSE;
    }
  }

  /**
   * Gets the content of the llms.txt file.
   *
   * @return string|false
   *   The content of the llms.txt file, or FALSE if it doesn't exist.
   */
  public function getLlmsFileContent() {
    $uri = self::LLMS_TXT_URI;

    if (file_exists($uri)) {
      return file_get_contents($uri);
    }

    // Attempt to generate the file if it doesn't exist.
    if ($this->generateLlmsFile()) {
      return file_get_contents($uri);
    }

    return FALSE;
  }

  /**
   * {@inheritDoc}
   */
  public function getDefaultTemplateContent() {
    $site_name = $this->configFactory->get('system.site')->get('name');
    $site_url = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost();
    $date = date('M d Y');

    // Get the module path.
    $module_path = $this->moduleHandler->getModule('llms_txt_generator')->getPath();
    $template_path = $module_path . '/templates/default-template.md';

    // Read the template file.
    $template = file_get_contents($template_path);

    // Replace placeholders.
    $template = str_replace('{site_name}', $site_name, $template);
    $template = str_replace('{site_url}', $site_url, $template);
    $template = str_replace('{date}', $date, $template);

    return $template;
  }

}
